# Generated by Django 3.2.5 on 2022-04-27 09:29

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ActivityLogModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('user_id', models.IntegerField()),
                ('api', models.CharField(help_text='API URL', max_length=1024)),
                ('url_path', models.CharField(max_length=512)),
                ('url_name', models.CharField(max_length=255)),
                ('headers', models.TextField()),
                ('body', models.TextField()),
                ('method', models.CharField(db_index=True, max_length=10)),
                ('client_ip_address', models.CharField(max_length=50)),
                ('response', models.TextField()),
                ('status_code', models.PositiveSmallIntegerField(db_index=True, help_text='Response status code')),
                ('execution_time', models.DecimalField(decimal_places=5, help_text='Server execution time (Not complete response time.)', max_digits=8)),
                ('created_time', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'verbose_name': 'Activity Log',
                'verbose_name_plural': 'Activity Logs',
                'db_table': 'drf_activity_log',
            },
        ),
    ]
